"""
================
Cleaning data
================
The following module demonstrates data cleaning procedures such as recoding, missing data handling,
standardization, and subsetting a small sample data for modeling.
"""


import pandas as pd
import numpy as np
from scipy.stats import zscore

def school_type(series):
    """In the 'School_type' variable there are multiple responses labeled as 'Invalid', or 'No response'.
    These will be coded along with system missing code '99' as 'NaN'. The 'public' and 'private' school type will be
    recoded as 1 and -1 respectively."""
    if series == 8:
        return 'NaN'
    elif series == '.I':
        return 'NaN'
    elif series == 9:
        return 'NaN'
    elif series == '.M':
        return 'NaN'
    elif series == 99:
        return 'NaN'
    elif series == 1:
        return 1
    elif series == 2:
        return -1
    else:
        raise KeyError()


def data_cleaning(df):
    # changing datatype
    df['IBTEACH'] = df.IBTEACH.astype(float)

    df['IBTEACH'] = df.IBTEACH.astype(float)
    df['WEALTH'] = df.WEALTH.astype(float)
    df['ESCS'] = df.ESCS.astype(float)
    df['SC013Q01TA'] = df.SC013Q01TA.astype(float)
    df['SCIERES'] = df.SCIERES.astype(float)

    # create log_score for each subject, using log_science for science subject
    df['log_science'] = np.log(df.Science)

    # rename column names
    df.rename(columns={'SC013Q01TA': 'School_type', 'SCIERES': 'Sch_science_resource'}, inplace=True)

    # replacing '99' as NaN in all columns
    df.loc[:, 'IBTEACH': 'Sch_science_resource'] = df.loc[:, 'IBTEACH': 'Sch_science_resource'].replace(99, np.NaN)
    df['School_type'] = df['School_type'].apply(school_type)
    df.dropna(inplace=True)
    # summarize the number of rows and columns in the dataset
    print(df.shape)
    print(df.isnull().sum())

    # creating "female" variable effect coded for each country, female = 1, male = -1
    df['female'] = df.Gender.map({'Female': 1, 'Male': -1})

    # creating z-scores for selected variables
    df['z_sch_resource'] = zscore(df['Sch_science_resource'])
    df['z_IBTEACH'] = zscore(df['IBTEACH'])
    return df.to_csv('data.csv', encoding='utf-8')


def small_sample(df):
    """output a random sample data csv file for smaller sample analysis"""
    df_s = df.groupby(['CountryID', 'SchoolID']).apply(lambda x: x.sample(frac=0.5, random_state=999)).\
        reset_index(drop=True)
    return df_s.to_csv('sample data.csv', encoding='utf-8')


def main():
    # df = pd.read_csv("data/student_info.csv", encoding='latin-1', na_values=['', ' '])
    df = pd.read_csv("student_info.csv", encoding='latin-1', na_values=['', ' '])
    data_cleaning(df)

    # df = pd.read_csv('data/data.csv', header=0)
    df = pd.read_csv('data.csv', header=0)
    small_sample(df)

if __name__ == '__main__':
    main()


# A grand function that takes as input a list of country codes, then uses our function above WB_country_data to
# get a list of indicators for each country,then plot a line diagram
def plot_indicators(country_list, indicator, start=2000, end=2015):
    import matplotlib.pyplot as plt
    import seaborn as sns
    import wbdata
    import re
    ind = wbdata.get_indicator(indicator, display=False)
    # capture the title which includes the unit after bracket
    title = ind[0]['name']
    # now take entire text from first letter to before opening bracket
    title = title[:title.find('(')-1]
    # this is the patter to match anything between two brackets
    p = re.compile('\((.*?)\)')
    ylab = p.findall(ind[0]['name'])[0]
    sns.set_style('white')
    fig, axis = plt.subplots()
    for c in country_list:
        axis.plot(range(start,end+1), country_data(c,indicator,start,end))
    plt.legend(country_list)
    plt.title(title)
    plt.ylabel(ylab)
    plt.show()


'''
CSE583 Project
This program merges student file or teacher file with school 
file accoding to 'CNTSCHID' and 'SUBNATIO'
'''

import pandas as pd

def merge(file, output):
    '''
    Merges file with school file according to 'CNTSCHID' and 'SUBNATIO',
    and generates a new output csv file
    '''
    school_file = 'Data files/PISA data/schools.csv'
    df = pd.read_csv(file)
    school_df = pd.read_csv(school_file)
    result = pd.merge(df, school_df, how = 'left',
                       on = ['CNTSCHID', 'SUBNATIO'])
    result.to_csv(output, index = False)
    
def main():
    teacher_file = 'Data files/PISA data/teachers.csv'
    teacher_output = 'teascho.csv'
    student_file = 'Data files/PISA data/students.csv'
    student_output = 'stuscho.csv'
    merge(teacher_file, teacher_output)
    merge(student_file, student_output)
    
if __name__ == '__main__':
    main()


import UnitTests
import pandas as pd
from support import *

class UnitTests(unittest.TestCase):

    def test_sav_to_dataframe(self):
        ''' Test for file not existing'''
        with self.assertRaises(FileNotFoundError):
            sav_to_dataframe('123.sav')
        ''' Test for file with a wrong type'''
        with self.assertRaises(FileNotFoundError):
            sav_to_dataframe('nations.csv')
        ''' Test for a valid file'''
        file = 'sample.sav'
        df = sav_to_dataframe(file)
        self.assertTrue(len(df) == 3)

    def test_student2school(self):
        df = pd.read_csv('IDs_sorted_by_student.csv')
        ''' Test for invalid student id'''
        with self.assertRaises(KeyError):
            student2school(df, 0)
        with self.assertRaises(KeyError):
            student2school(df, 99999999)
        ''' Test for valid student id'''
        nID = student2school(df, 800001)
        self.assertTrue(nID == 80000)

    def test_school2nation(self):
        ''' Test for id less than 97100000'''
        nID1 = school2nation(3600227)
        self.assertTrue(nID1 == 360000)
        nID2 = school2nation(7610535)
        self.assertTrue(nID2 == 760000)
        ''' Test for id greater than 97100000 and less than 97200000'''
        nID3 = school2nation(97100500)
        self.assertTrue(nID3 == 7240000)
        ''' Test for id greater than 97200000 and less than 97400000'''
        nID4 = school2nation(97311281)
        self.assertTrue(nID4 == 8400000)
        ''' Test for id greater than 97400000'''
        nID5 = school2nation(97400091)
        self.assertTrue(nID5 == 320100)

    def test_student2nation(self):
        ''' Test for id less than 97100000'''
        nID1 = student2nation(3600227)
        self.assertTrue(nID1 == 360000)
        nID2 = student2nation(7610535)
        self.assertTrue(nID2 == 760000)
        ''' Test for id greater than 97100000 and less than 97200000'''
        nID3 = student2nation(97100500)
        self.assertTrue(nID3 == 7240000)
        ''' Test for id greater than 97200000 and less than 97400000'''
        nID4 = student2nation(97311281)
        self.assertTrue(nID4 == 8400000)
        ''' Test for id greater than 97400000'''
        nID5 = student2nation(97400091)
        self.assertTrue(nID5 == 320100)
    
    def test_info(self):
        df = pd.read_csv('nations.csv')
        ''' Test for invalid nation id'''
        with self.assertRaises(KeyError):
            info(df, 0)
        ''' Test for valid nation id'''
        res = info(df, 360000)
        self.assertTrue(res == ('AUS', 'AP'))

if __name__ == ‘__main__’:
    unittest.main()



# Scatterplot Matrix
import matplotlib.pyplot as plt
import pandas as pd
import numpy as np
from pandas.plotting import scatter_matrix


def calculate_correlation(df, colnames):
    # dataframe for plotting
    df_plot = df.loc[:, list(colnames)]
    correlations = df_plot.corr()
    # plot correlation matrix
    names = colnames
    fig = plt.figure()
    ax = fig.add_subplot(111)
    cax = ax.matshow(correlations, vmin=-1, vmax=1)
    fig.colorbar(cax)
    ticks = np.arange(0,9,1)
    ax.set_xticks(ticks)
    ax.set_yticks(ticks)
    ax.set_xticklabels(names)
    ax.set_yticklabels(names)
    plt.show()

def visual_scatterplot(df, colnames):
    # Scatterplot Matrix
    df_plot = df.loc[:, list(colnames)]
    scatter_matrix(df_plot)
    plt.show()

def visual_histplot(df, colnames):
    # histogram plot
    df_plot = df.loc[:, list(colnames)]
    df_plot.hist()
    plt.show()

def visual_densityplot(df, colnames):
    # Univariate density plot, as a way to look at distribution
    df_plot = df.loc[:, list(colnames)]
    df_plot.plot(kind='density', subplots=True, layout=(4,4), sharex=False)
    plt.show()


def main():
    df = pd.read_csv('sample data.csv', header=0)
    colnames = ('Science', 'IBTEACH', 'WEALTH', 'ESCS', 'School_type',
                'Sch_science_resource', 'log_science', 'female')
    calculate_correlation(df, colnames)
    visual_histplot(df, colnames)
    visual_densityplot(df, colnames)

if __name__ == '__main__':
    main()


import UnitTests
import matplotlib.pyplot as plt
import pandas as pd
import numpy as np
from pandas.plotting import scatter_matrix
from correlation import *

class UnitTests(unittest.TestCase):
    df = pd.read_csv('sample data.csv', header=0)
    colnames = ('Science', 'IBTEACH', 'WEALTH', 'ESCS', 'School_type',
                'Sch_science_resource', 'log_science', 'female')
                
    def smoke_test_calculate_correlation(self):
        ''' Smoke test'''
        try:
            calculate_correlation(df, colnames)
        except:
            print('Smoke test failed!')
        print('Smoke test passed!')

    def smoke_test_visual_scatterplot(self):
        ''' Smoke test'''
        try:
                visual_scatterplot(df, colnames)
        except:
            print('Smoke test failed!')
        print('Smoke test passed!')

    def smoke_test_visual_histplot(self):
        ''' Smoke test'''
        try:
            visual_histplot(df, colnames)
        except:
            print('Smoke test failed!')
        print('Smoke test passed!')

    def smoke_test_visual_densityplot(self):
        ''' Smoke test'''
        try:
            visual_densityplot(df, colnames)
        except:
            print('Smoke test failed!')
        print('Smoke test passed!')

if __name__ == ‘__main__’:
    unittest.main()



# function for specific country lookup

def country_data(country_code, indicator, start=2015, end=2015):
    import datetime
    import wbdata
    data_dates = (datetime.datetime(start, 1, 1), datetime.datetime(end, 1, 1))
    # call the api
    data = wbdata.get_dataframe({indicator: 'indicator'},
                                country=country_code,
                                data_date=data_dates,
                                convert_date=True,
                                keep_levels=False)

    data = data.reset_index()
    # data = data.dropna() #if I want I can drop the na's
    return data[['indicator']]


# function for getting gender indicator data for all countries
def WB_country_data(indicator, start=2015, end=2015):
    import datetime
    import wbdata
    data_dates = (datetime.datetime(start, 1, 1), datetime.datetime(end, 1, 1))
    # call the api
    data = wbdata.get_dataframe({indicator: 'indicator'},
                                data_date=data_dates,
                                convert_date=True,
                                keep_levels=False)

    data = data.reset_index()
    # data = data.dropna() #if I want I can drop the na's
    return data[['indicator']]



'''
CSE583 Project
Given a student file, extracts useful features and writes
as a csv file
'''

import pandas as pd
from support import infos, student2nation

def get_student_info():
    '''
    Computes and extracts useful data from student-school datafile,
    stores data into a new csv file
    '''
    # Reads csv files and creates dataframe
    df = pd.read_csv('Data files/PISA data/small_stuscho.csv')
    country_dict_df = pd.read_csv('Data files/PISA data/nations.csv')
    
    # Dataframe processing
    df1 = df[['CNTRYID', 'CNTSCHID', 'CNTSTUID', 'ST004D01T']]
    df1.columns = ['CountryID', 'SchoolID', 'StudentID', 'Gender']
    stuid_list = df1['StudentID'].tolist()
    countryid_list = []
    for stuid in stuid_list:
        countryid_list.append(student2nation(stuid))
    df_country = infos(country_dict_df, countryid_list)
    df_math = df.iloc[:, 809:819]
    df_math['Mathematics'] = df_math.mean(axis=1)
    df_read = df.iloc[:, 819:829]
    df_read['Reading'] = df_read.mean(axis=1)
    df_science = df.iloc[:, 829:839]
    df_science['Science'] = df_science.mean(axis=1)
    df_hlm = df[['IBTEACH', 'WEALTH', 'ESCS', 'SC013Q01TA', 'SCIERES']]
    
    output_df = pd.concat([df1, df_country, df_math['Mathematics'],
                           df_read['Reading'], df_science['Science'],
                           df_hlm], axis=1, sort=False)
    output_df.loc[df1['Gender'] == 1, 'Gender'] = 'Female'
    output_df.loc[df1['Gender'] == 2, 'Gender'] = 'Male'
    output_df.to_csv('small_student_info.csv', encoding='utf-8')
    
def main():
    get_student_info()
    
if __name__ == '__main__':
    main()



import UnitTests
import pandas as pd
import numpy as np
from scipy.stats import zscore
from data_cleaning import *

class UnitTests(unittest.TestCase):
    
    def test_school_type(self):
        ''' Test for valid variable'''
        series1 = pd.Series(['.I', '.M', 8, 9, 99, 1, 2])
        res1 = school_type(series1)
        self.assertTrue(res1 == pd.Series(['NaN', 'NaN', 'NaN', 'NaN', 'NaN', 1, -1]))
        ''' Test for invalid variable'''
        series2 = pd.Series(['invalid', 10])
        with self.assertRaises(KeyError):
            res2 = school_type(series2)

    def smoke_test_data_cleaning(self):
        ''' Smoke test'''
        df = pd.read_csv("student_info.csv", encoding='latin-1', na_values=['', ' '])
        try:
            data_cleaning(df)
        except:
            print('Smoke test failed!')
        print('Smoke test passed!')

    def smoke_test_small_samle(self):
        ''' Smoke test'''
        df = pd.read_csv('data.csv', header=0)
        try:
            small_sample(df)
        except:
            print('Smoke test failed!')
        print('Smoke test passed!')

if __name__ == ‘__main__’:
    unittest.main()



"""
=======================================
Fitting and plotting multi-level models
=======================================
This module runs pymer4 package which mimics functionality of lmer4 in R.
Here each model category is exemplified with a selection of variables in
the PISA 2015 dataset and we will show a demonstration of model fitting with regard to
gender (female) and a group of other interested factors. The results show the model
coefficients for each model.
"""


import os
import pandas as pd
import seaborn as sns
from pymer4.models import Lmer
from pymer4.utils import get_resource_path

###############################################################################
# Fitting hierarchical level mixed effects models
# ---------------
#
def random_intercept_3level_model(df):
    # Random Intercept-only three-level model
    model_0_sci = Lmer('log_science ~ 1 | SchoolID/CountryID', data=df)
    model_0_sci.fit(REML=False)
    m0_list=model_0_sci.fit(REML=False)
    model_0_sci.coefs


# Calculating ICC

def random_effect_2level_model(df):
    # Random intercept and slop two-level model:
    model_1_sci = Lmer('Science ~ female + (female*ESCS | CountryID)', data=df)
    model_1_sci.fit(REML=False)
    m1_list = model_1_sci.fit(REML=False)
    model_1_sci.coefs


def fixed_effect_3level_model(df):
    # Fixed effects three-level model
    model_2_sci = Lmer('log_science ~ IBTEACH + WEALTH '
                       '+ ESCS + female + Sch_science_resource '
                       '+ (1 | SchoolID/CountryID)',
                       data=df)
    model_2_sci.fit(REML=False)
    m2_list = model_2_sci.fit(REML=False)


def fixeff_interaction_3level_model(df):
    # Fixed effects plus interaction three-level model
    model_3_sci = Lmer('log_science ~ IBTEACH + WEALTH '
                       '+ ESCS + female + Sch_science_resource'
                       ' + female*ESCS '
                       '+ (1 | SchoolID/CountryID)',
                       data=df)
    model_3_sci.fit(REML=False)
    m3_list = model_3_sci.fit(REML=False)


def mixeff_interaction_3level_model(df):
    # Mixed effects plus interaction three-level model
    model_4_sci = Lmer('log_science ~ IBTEACH + WEALTH '
                       '+ ESCS + female + Sch_science_resource '
                       '+ female*ESCS '
                       '+ (ESCS | CountryID) '
                       '+ (Sch_science_resource | SchoolID/CountryID) '
                       '+ (1 | SchoolID/CountryID)',
                       data=df)
    model_4_sci.fit(REML=False)
    m4_list = model_4_sci.fit(REML=False)


def random_effect_multinteraction_model(df):
    # one random effect and multiple interactions between gender and factors
    model_5_sci = Lmer('log_science ~ IBTEACH + WEALTH + ESCS + female + Sch_science_resource + female*ESCS '
                       '+ female*WEALTH + female*IBTEACH + (ESCS | CountryID)',
                       data=df)
    model_5_sci.fit(REML=False)
    m5_list = model_5_sci.fit(REML=False)


# Model fit
# BIC change (of main effects combined M0 to M1
# BIC change (of main effects combined M1 to M2
# Using a likelihood ratio test (LRT) to test the difference between any two models
# Calculate the change in approximate variance explained (approximate R2) from M0 to M1, and M1 to M2.

###############################################################################
# Visualizing results
# -------------------
#
# Visualize model summary
def model_sum_visual(model_x_sci, predictor):
    model_x_sci.plot_summary()
    # Visualizing random effect of a predictor
    model_x_sci.plot('predictor',
               plot_ci=True,
               ylabel='Predicted log_science')

    sns.regplot(x='predictor',
                y='residuals',
                data=model_x_sci.data,
                fit_reg= False
                )
    # Inspecting overall fit
    sns.regplot(x='fits',
                y='log_science',
                units='CountryID',
                data=model_x_sci.data,
                fit_reg=True
                )

###############################################################################
# Main: Loading data
# ------------
#
def main():
    # Reading PISA csv file
    # df = pd.read_csv('data.csv')
    df = pd.read_csv('sample data.csv')
    random_effect_multinteraction_model(df)
    model_sum_visual(model_5_sci, ESCS)


if __name__ == '__main__':
    main()



from distutils.core import setup

setup(
    name='PISAnalysisTool',
    version='0.1.0',
    author='Nixi Wang',
    author_email='nixiwang@uw.edu',
    packages=['PISAnalysisTool', 'PISAnalysisTool.tests'],
    url='http://pypi.python.org/pypi/PISAnalysisTool/',
    license='LICENSE.txt',
    description='An analysis tool of multi-level modeling for PISA dataset',
    long_description=open('README.md').read(),
    install_requires=[
      "appdirs==1.4.3",
      "appnope==0.1.0",
      "asn1crypto==0.24.0",
      "astroid==2.0.4",
      "attrs==18.2.0",
      "Automat==0.7.0",
      "backcall==0.1.0",
      "bleach==3.0.0",
      "certifi==2018.10.15",
      "cffi==1.11.5",
      "chardet==3.0.4",
      "conda==4.5.11",
      "constantly==15.1.0",
      "cryptography==2.3.1",
      "cycler==0.10.0",
      "decorator==4.3.0",
      "entrypoints==0.2.3",
      "h5py==2.8.0",
      "html5lib==1.0.1",
      "hyperlink==18.0.0",
      "idna==2.7",
      "incremental==17.5.0",
      "ipykernel==5.0.0",
      "ipython==7.0.1",
      "ipython-genutils==0.2.0",
      "ipywidgets==7.4.2",
      "isort==4.3.4",
      "jedi==0.13.1",
      "Jinja2==2.10",
      "joblib==0.13.0",
      "jsonschema==2.6.0",
      "jupyter==1.0.0",
      "jupyter-client==5.2.3",
      "jupyter-console==6.0.0",
      "jupyter-core==4.4.0",
      "kiwisolver==1.0.1",
      "lazy-object-proxy==1.3.1",
      "MarkupSafe==1.0",
      "matplotlib==3.0.0",
      "mccabe==0.6.1",
      "mistune==0.8.3",
      "mkl-fft==1.0.6",
      "mkl-random==1.0.1",
      "nbconvert==5.3.1",
      "nbformat==4.4.0",
      "notebook==5.7.0",
      "numpy==1.15.2",
      "pandas==0.23.4",
      "pandocfilters==1.4.2",
      "parso==0.3.1",
      "patsy==0.5.1",
      "pexpect==4.6.0",
      "pickleshare==0.7.5",
      "prometheus-client==0.4.0",
      "prompt-toolkit==2.0.5",
      "ptyprocess==0.6.0",
      "pyasn1==0.4.4",
      "pyasn1-modules==0.2.2",
      "pycosat==0.6.3",
      "pycparser==2.19",
      "Pygments==2.2.0",
      "pylint==2.1.1",
      "pymer4==0.6.0",
      "pyOpenSSL==18.0.0",
      "pyparsing==2.2.2",
      "PySocks==1.6.8",
      "python-dateutil==2.7.3",
      "pytz==2018.5",
      "pyzmq==17.1.2",
      "qtconsole==4.4.1",
      "requests==2.19.1",
      "rpy2==2.9.4",
      "ruamel-yaml==0.15.46",
      "scipy==1.1.0",
      "seaborn==0.9.0",
      "Send2Trash==1.5.0",
      "service-identity==17.0.0",
      "simplegeneric==0.8.1",
      "six==1.11.0",
      "terminado==0.8.1",
      "testpath==0.4.2",
      "tornado==5.1.1",
      "tqdm==4.28.1",
      "traitlets==4.3.2",
      "Twisted==17.5.0",
      "typed-ast==1.1.0",
      "tzlocal==1.5.1",
      "urllib3==1.23",
      "wbdata==0.2.7",
      "wcwidth==0.1.7",
      "webencodings==0.5.1",
      "widgetsnbextension==3.4.2",
      "wrapt==1.10.11",
      "zope.interface==4.5.0"
    ],
)


import UnitTests
from pymer4.models import Lmer
from pymer4.utils import get_resource_path
import pandas as pd
import numpy as np
import os
from pymer4 import *

class UnitTests(unittest.TestCase):
    def test_hlm_m0():

        df = pd.read_csv(os.path.join(get_resource_path(), 'sample data.csv'))
        model_0_sci = Lmer('log_science ~ 1 | SchoolID/CountryID', data=df)
        model_0_sci.fit(summarize=False, REML=False)

        # Test results against r output
        assert model_0_sci.coefs.shape == (1, 8)
        estimates = np.array([6.138196])
        assert np.allclose(model_0_sci.coefs['Estimate'], estimates, atol=.001)

        assert isinstance(model_0_sci.fixef, list)
        assert model_0_sci.fixef[0].shape == (14419, 1)
        assert model_0_sci.fixef[1].shape == (14419, 1)

        assert isinstance(model_0_sci.ranef, list)
        assert model_0_sci.ranef[0].shape == (14419, 1)
        assert model_0_sci.ranef[1].shape == (14419, 1)

        assert model_0_sci.ranef_var.shape == (3, 3)

        variance = np.array([0.008007, 0.014925, 0.021602])
        assert np.allclose(
            model_0_sci.ranef_var.loc[:, 'Var'], variance, atol=.01)


    def test_hlm_m1():

        df = pd.read_csv(os.path.join(get_resource_path(), 'sample data.csv'))
        model_1_sci = Lmer('Science ~ female + (female*ESCS | CountryID)',
                           data=df)
        model_1_sci.fit(summarize=False, REML=False)

        # Test results against r output
        assert model_1_sci.coefs.shape == (2, 8)
        estimates = np.array([412.298572, 6.220405])
        assert np.allclose(model_1_sci.coefs['Estimate'], estimates, atol=.001)

        assert model_1_sci.ranef_var.shape == (5, 3)

    def test_hlm_m2():

        df = pd.read_csv(os.path.join(get_resource_path(), 'sample data.csv'))
        model_2_sci = Lmer('log_science ~ IBTEACH + WEALTH '
                           '+ ESCS + female + Sch_science_resource '
                           '+ (1 | SchoolID/CountryID)',
                           data=df)
        model_2_sci.fit(summarize=False, REML=False)

        # Test results against r output
        assert model_2_sci.coefs.shape == (6, 8)
        estimates = np.array([6.074618, -0.015097, 0.010710, 0.042099, -0.006254, 0.016298])
        assert np.allclose(model_2_sci.coefs['Estimate'], estimates, atol=.001)

        assert isinstance(model_2_sci.fixef, list)
        assert model_2_sci.fixef[0].shape == (14419, 6)
        assert model_2_sci.fixef[1].shape == (14419, 6)

        assert isinstance(model_2_sci.ranef, list)
        assert model_2_sci.ranef[0].shape == (14419, 1)
        assert model_2_sci.ranef[1].shape == (14419, 1)

        assert model_2_sci.ranef_var.shape == (3, 3)

        assert np.allclose(
            model_2_sci.coefs.loc[:, 'Estimate'], model_2_sci.fixef[0].mean(), atol=.01)


    def test_hlm_m3():

        df = pd.read_csv(os.path.join(get_resource_path(), 'sample data.csv'))
        model_3_sci = Lmer('log_science ~ IBTEACH + WEALTH '
                           '+ ESCS + female + Sch_science_resource '
                           '+ female*ESCS '
                           '+ (1 | SchoolID/CountryID)',
                           data=df)
        model_3_sci.fit(summarize=False, REML=False)

        # Test results against r output
        assert model_3_sci.coefs.shape == (3, 8)
        estimates = np.array([6.074593, -0.015087, -0.010693, 0.042019, -0.006004, 0.016301, 0.000859])
        assert np.allclose(model_3_sci.coefs['Estimate'], estimates, atol=.001)

        assert isinstance(model_3_sci.fixef, list)
        assert model_3_sci.fixef[0].shape == (14419, 7)
        assert model_3_sci.fixef[1].shape == (14419, 7)

        assert isinstance(model_3_sci.ranef, list)
        assert model_3_sci.ranef[0].shape == (14419, 1)
        assert model_3_sci.ranef[1].shape == (14419, 1)

        assert model_3_sci.ranef_var.shape == (3, 3)

        assert np.allclose(
            model_3_sci.coefs.loc[:, 'Estimate'], model_3_sci.fixef[0].mean(), atol=.01)


    def test_hlm_m4():

        df = pd.read_csv(os.path.join(get_resource_path(), 'sample data.csv'))
        model_4_sci = Lmer('log_science ~ IBTEACH + WEALTH '
                           '+ ESCS + female + Sch_science_resource '
                           '+ female*ESCS '
                           '+ (Sch_science_resource | SchoolID/CountryID) '
                           '+ (ESCS | CountryID) '
                           '+ (1| SchoolID/CountryID)',
                           data=df)
        model_4_sci.fit(summarize=False, REML=False)

        # Test results against r output
        assert model_4_sci.coefs.shape == (7, 8)
        estimates = np.array([6.079710, -0.013440, -0.012672, 0.046351,
                              -0.005399, 0.013502, 0.000887])
        assert np.allclose(model_4_sci.coefs['Estimate'], estimates, atol=.001)

        assert isinstance(model_4_sci.fixef, list)
        assert model_4_sci.fixef[0].shape == (14419, 7)
        assert model_4_sci.fixef[1].shape == (14419, 7)

        assert isinstance(model_4_sci.ranef, list)
        assert model_4_sci.ranef[0].shape == (14419, 3)
        assert model_4_sci.ranef[1].shape == (14419, 3)

        assert model_4_sci.ranef_corr.shape == (3, 3)
        assert model_4_sci.ranef_var.shape == (9, 3)

        assert np.allclose(
            model_4_sci.coefs.loc[:, 'Estimate'], model_4_sci.fixef[0].mean(), atol=.01)


    def test_hlm_m5():

        df = pd.read_csv(os.path.join(get_resource_path(), 'sample data.csv'))
        model_5_sci = Lmer('log_science ~ IBTEACH + WEALTH '
                           '+ ESCS + female + Sch_science_resource '
                           '+ female*ESCS + female*WEALTH + female*IBTEACH '
                           '+ (ESCS | CountryID)',
                   data=df)
        model_5_sci.fit(summarize=False, REML=False)

        # Test results against r output
        assert model_5_sci.coefs.shape == (9, 8)
        estimates = np.array([6.108262, -0.016521, -0.013483, 0.074545,
                              0.000658, 0.010694,
                              -0.000788, 0.001784, 0.003985])
        assert np.allclose(model_5_sci.coefs['Estimate'], estimates, atol=.001)

        assert isinstance(model_5_sci.fixef, list)
        assert model_5_sci.fixef.shape == (64, 9)

        assert isinstance(model_5_sci.ranef, list)
        assert model_5_sci.ranef.shape == (64, 2)

        assert model_5_sci.ranef_corr.shape == (1, 3)
        assert model_5_sci.ranef_var.shape == (3, 3)

        assert np.allclose(
            model_5_sci.coefs.loc['(Intercept)', 'Estimate'], model_5_sci.fixef['(Intercept)'].mean(), atol=.01)


if __name__ == ‘__main__’:
    unittest.main()


from .merge import __main__
from .student import __main__
from .data_cleaning import __main__
from .correlation import __main__
from .hlm_pymer4 import __main__


__all__ = ["merge",
           "student",
           "data_cleaning", "correlation", "hlm_pymer4",
           "__version__"]



'''
    CSE583 Project
    Supporting functions
'''
import pandas as pd
import savReaderWriter as spss
from pathlib import Path

def sav_to_dataframe(file):
    if not os.path.isfile(file):
        print('File not exists!')
        raise FileNotFoundError()
    
    if not file.endswith('.sav'):
        print('It is not a sav file!')
        raise FileNotFoundError()

    records = []
    with spss.SavReader(file) as reader:
        print("Reading file:",file, "...")
        for line in reader:
            records.append(line)

    df = pd.DataFrame(records)
    return df


def student2school(df, id):
    '''
        Takes the 'IDs_sorted_by_student.csv' dataframe and student id, returns school id of the student.
        '''
    return student2schoolHelper(df, id, 0, len(df))


def student2schoolHelper(df, id, start, end):
    if start > end:
        print('Student ID:' + id + ' not exists!')
        raise KeyError()
    elif start == end:
        if id == student2schoolHelper['CNTSCHID'][start]:
            return student2schoolHelper['SUBNATIO'][start]
        else:
            print('Student ID:' + id + ' not exists!')
            raise KeyError()
    index = (end + start) // 2
    mid_value = df['CNTSCHID'][index]
    if mid_value == id:
        return df['SUBNATIO'][index]
    elif mid_value > id:
        return student2schoolHelper(df, id, start, index)
    else:
        return student2schoolHelper(df, id, index, end)


def school2nation(id):
    '''
        Takes school id, returns nation id of the school.
        '''
    if id < 97100000:
        return id // 100000 * 10000
    if id < 97200000:
        return 7240000
    if id < 97400000:
        return 8400000
    return 320100

def student2nation(id):
    '''
        Takes student id, returns nation id of the student.
        '''
    return school2nation(id)


def info(df, id):
    '''
    :param df: the dataframe of 'nations.csv'
    :param id: the id of nation
    :return: a tuple - (string country name, string country code, string continent)
    '''
    try:
        target = df[df['nationID'] == id]
    except KeyError:
        print('Nation ID:' + id + ' not exists!')
        raise KeyError()

    name = target['nationName'].tolist()
    code = target.nationCode.tolist()
    continent = target.continent.tolist()

    return (name[0], code[0], continent[0])


def infos(country_df, id_list):
    '''
    :param country_df: the dataframe of 'nations.csv'
    :param id_df: the dataframe of country id
    :return: a dataframe - (string country name, string country code, string continent)
    '''

    name_list = []
    continent_list = []
    code_list = []

    for i in range(len(id_list)):
        t = info(country_df, id_list[i])
        name_list.append(t[0])
        code_list.append(t[1])
        continent_list.append(t[2])

    dict = {'CountryName':name_list, 'CountryCode': code_list, 'Continent':continent_list}
    res = pd.DataFrame(dict)

    return res
